/* Programma per il calcolo del perimetro di una forma geometrica */

#include <stdio.h>
#include <string.h>

/* Definizione delle costanti */
const int TRUE=1;
const int FALSE=0;

int verifica_arg(char *stringa);
float perimetro_poligono(int lati, char *vertici[]);
float lunghezza_lato(int v1x, int v1y, int v2x, int v2y);
void estrai(char *vertice1,char *vertice2,int *asc1v, int *ord1v, int *asc2v, int *ord2v);

int main(int argc, char *argv[])
{ float perim;
  char *forma;

  /* Controllo di validita' dei parametri passati da riga di comando */
  if (argc<2)
    { printf("\nERRORE: Il programma e\' stato eseguito senza parametri.\n\n");
      printf("Sintassi:\n\tperimetro X1,Y1 X2,Y2 X3,Y3 ...\n");
      printf("Esempio: perimetro 2,2 10,2 13,7 1,1\n");
      return 1;
    }
  else if (argc<4)
    { printf("\nERRORE: Numero di parametri insufficienti.\n");
      printf("Sono stati inseriti %d parametri; bisogna inserirne almeno 3 e massimo 10.\n", argc-1);
      return 1;
    }
  else if (argc>11)
    { printf("\nERRORE: Numero di parametri eccessivi.\n");
      printf("Sono stati inseriti %d parametri; bisogna inserirne almeno 3 e massimo 10.\n", argc-1);
      return 1;
    }
  else
    { /* Controllo di sintassi della riga di comando */
      int i, errore=FALSE;
      for (i=1;i<argc;i++)
        { if (verifica_arg(argv[i])==0)
            { printf("\nErrore di sintassi nel %d parametro.",i);
              errore=TRUE;
            }
        };
      if (errore)
        { printf("\nSintassi:\n\tperimetro X1,Y1 X2,Y2 X3,Y3 ...\n");
          printf("Esempio: perimetro 2,2 10,2 13,7 1,1\n");
          return 1;
        };
    };

  /* Esecuzione del programma */
  perim=perimetro_poligono(argc-1,argv);
  switch (argc-1)
  {  case 3:
       strcpy(forma, "triangolo");
       break;
     case 4:
        strcpy(forma, "quadrilatero");
        break;
     case 5:
        strcpy(forma, "pentagono");
        break;
     case 6:
        strcpy(forma, "esagono");
        break;
    default:
        strcpy(forma, "poligono");
 };
  printf("\n\nLa figura geometrica corrisponde ad un %s.",forma);
  printf("\nIl perimetro del %s e\' pari a: %8.3f\n",forma,perim);
  
  return 0;
}

/* Funzione che verifica la correttezza sistattica dei parametri forniti */
int verifica_arg(char *stringa)
{ int i=0;
  int corretto=TRUE;
  
  if (isdigit(stringa[0]))
    while (isdigit(stringa[i])) i++;
  else corretto=FALSE;
  
  if ((corretto)&&(stringa[i]==','))
    i++;
  else corretto=FALSE;
  
  if (isdigit(stringa[i]))
    while (isdigit(stringa[i])) i++;
  else corretto=FALSE;
  
  if (stringa[i]!='\0') corretto=FALSE;
  return corretto;
}

/* Calcolo del perimetro del poligono */
float perimetro_poligono(int lati, char *vertici[])
{ float perimetro=0;
  int i=1;
  int vert1x, vert1y, vert2x, vert2y;
  char argom1[10], argom2[10];
  while (lati>0)
  { lati--;
    if (lati==0)
    { strcpy(argom1,vertici[i]);
      strcpy(argom2,vertici[1]);
      estrai(argom1, argom2, &vert1x, &vert1y, &vert2x, &vert2y);
    }
    else
    { strcpy(argom1,vertici[i]);
      strcpy(argom2,vertici[i+1]);
      estrai(argom1, argom2, &vert1x, &vert1y, &vert2x, &vert2y);
    };
    perimetro+=lunghezza_lato(vert1x, vert1y, vert2x, vert2y);
    i++;
  };
  return perimetro;
}

/* Calcola la lunghezza di un lato date le coordinate dei due vertici */
float lunghezza_lato(int v1x, int v1y, int v2x, int v2y)
{  int sommax, sommay;
   float risultato;
   sommax=v1x-v2x;
   sommay=v1y-v2y;
   sommax*=sommax;
   sommay*=sommay;
   risultato=sqrt(sommax+sommay);
   return risultato;
}

/* Estrae da due vertici le loro coordinate */
void estrai(char *vertice1,char *vertice2,int *asc1v, int *ord1v, int *asc2v, int *ord2v)
{ int coordinata,i,j;

  /* Ascissa del primo vertice */
    coordinata=0;
    i=0;
    while (isdigit(vertice1[i]))
      { coordinata*=10;
        coordinata+=(vertice1[i]-'0');
        i++;
      };
    *asc1v=coordinata;
 
   /* Ordinata del primo vertice */
    coordinata=0;
    i++;
    while (isdigit(vertice1[i]))
      { coordinata*=10;
        coordinata+=(vertice1[i]-'0');
        i++;
      };
    *ord1v=coordinata;
 
   /* Ascissa del secondo vertice */
    coordinata=0;
    i=0;
    while (isdigit(vertice2[i]))
      { coordinata*=10;
        coordinata+=(vertice2[i]-'0');
        i++;
      };
    *asc2v=coordinata;
 
   /* Ordinata del secondo vertice */
    coordinata=0;
    i++;
    while (isdigit(vertice2[i]))
      { coordinata*=10;
        coordinata+=(vertice2[i]-'0');
        i++;
      };
    *ord2v=coordinata;
}


